import numpy as np
import cv2
import tensorflow as tf
import random
import math

def generate_bitmap_labels(width, height, bboxes):
    bitmap_image = np.zeros([height, width, 1], dtype=np.float32)
    for bb in bboxes:
        bitmap_image[bb[1]:bb[3], bb[0]:bb[2]] = 1.0
    return bitmap_image

def generate_view_frames(width, height, size, stride):
    xc = -size[0]//3
    yc = -size[1]//3
    data = []
    while(yc+size[1]+stride[1] < (height+(size[1]//3))):
        while(xc+size[0]+stride[0] < (width+(size[0]//3))):
            data.append([xc, yc])
            xc += stride[0]
        yc += stride[1]
        xc = 0
    return data

def draw_confidence_map(image, confidence, stride, size):
    #print("drawing confidence map")
    cpy = image.copy()
    x_rem = image.shape[0]%stride
    y_rem = image.shape[1]%stride
    blank = np.zeros([image.shape[0]+(stride+x_rem)*2, image.shape[1]+((stride+y_rem)*2), 3], dtype=np.float32)
    blank[stride+x_rem:image.shape[0]+stride+x_rem, stride+y_rem:image.shape[1]+stride+y_rem, 0] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[stride+x_rem:image.shape[0]+stride+x_rem, stride+y_rem:image.shape[1]+stride+y_rem, 1] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[stride+x_rem:image.shape[0]+stride+x_rem, stride+y_rem:image.shape[1]+stride+y_rem, 2] = cpy.reshape([image.shape[0], image.shape[1]])
    blank2 = np.zeros([image.shape[0] + (stride + x_rem) * 2, image.shape[1] + ((stride + y_rem) * 2), 3], dtype=np.float32)
    confidence_map = confidence.reshape([math.ceil(image.shape[0]/stride), math.ceil(image.shape[1]/stride), 2])
    for i in range(confidence_map.shape[0]):
        for k in range(confidence_map.shape[1]):
            if confidence_map[i][k][0] > confidence_map[i][k][1] and confidence_map[i][k][1] < 0.1 and confidence_map[i][k][0] > 0.95:
                ix = k*stride
                kx = i*stride
                diff =  float(confidence_map[i][k][0] - confidence_map[i][k][1])
                try:
                    blank2[kx:kx+size, ix, 1] += diff
                    blank2[kx:kx+size, ix+size, 1] += diff
                    blank2[kx, ix:ix+size, 1] += diff
                    blank2[kx+size, ix:ix+size, 1] += diff
                except:
                    diff = 0.0

                #cv2.rectangle(blank2, (kx, ix), (kx+size, ix+size), color=(0, diff, 0), thickness=1, lineType=8, shift=0)
    cv2.add(blank, blank2, blank2)
    return blank2

def generate_patch_location(image_shape, stride, x_rem, y_rem):
    """Returns a 2d coordinate array of the top left x y locations for each patch."""
    #x_rem = -20
    #y_rem = -16
    width = math.ceil(image_shape[1]//stride)
    height = math.ceil(image_shape[0]//stride)
    #X, Y = np.mgrid[-x_rem:width-x_rem:stride, -y_rem:height:stride]
    #xy = np.vstack((X.flatten(), Y.flatten())).T
    xy = []
    for k in range(height):
        for i in range(width):
            xy.append([y_rem, x_rem])
            x_rem += stride
        y_rem += stride
        x_rem = -20

    return xy

def draw_patches_on_image(image, indexes, size, color):
    #print("drawing confidence map")

    cpy = image.copy()
    blank = np.zeros([image.shape[0], image.shape[1], 3], dtype=np.float32)
    blank[0:image.shape[0], 0:image.shape[1], 0] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[0:image.shape[0], 0:image.shape[1], 1] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[0:image.shape[0], 0:image.shape[1], 2] = cpy.reshape([image.shape[0], image.shape[1]])

    blank2 = np.zeros([image.shape[0], image.shape[1], 3], dtype=np.float32)
    for dp in indexes:
        kx=dp[0]
        ix=dp[1]
        diff = 0.0
        try:
            blank2[kx:kx + size, ix, 0] += color[0]
            blank2[kx:kx + size, ix + size, 0] += color[0]
            blank2[kx, ix:ix + size, 0] += color[0]
            blank2[kx + size, ix:ix + size, 0] += color[0]

            blank2[kx:kx + size, ix, 1] += color[1]
            blank2[kx:kx + size, ix + size, 1] += color[1]
            blank2[kx, ix:ix + size, 1] += color[1]
            blank2[kx + size, ix:ix + size, 1] += color[1]

            blank2[kx:kx + size, ix, 2] += color[2]
            blank2[kx:kx + size, ix + size, 2] += color[2]
            blank2[kx, ix:ix + size, 2] += color[2]
            blank2[kx + size, ix:ix + size, 2] += color[2]

        except:
            diff = 0.0

    cv2.add(blank, blank2, blank2)
    return blank2

def draw_bbx(image, bbx, color):
    #print("drawing confidence map")

    cpy = image.copy()
    blank = np.zeros([image.shape[0], image.shape[1], 3], dtype=np.float32)
    blank[0:image.shape[0], 0:image.shape[1], 0] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[0:image.shape[0], 0:image.shape[1], 1] = cpy.reshape([image.shape[0], image.shape[1]])
    blank[0:image.shape[0], 0:image.shape[1], 2] = cpy.reshape([image.shape[0], image.shape[1]])

    blank2 = np.zeros([image.shape[0], image.shape[1], 3], dtype=np.float32)

    try:
        blank2[bbx[0]:bbx[2], bbx[1], 0] += color[0]
        blank2[bbx[0]:bbx[2], bbx[3], 0] += color[0]
        blank2[bbx[0], bbx[1]:bbx[3], 0] += color[0]
        blank2[bbx[2], bbx[1]:bbx[3], 0] += color[0]

        blank2[bbx[0]:bbx[2], bbx[1], 1] += color[1]
        blank2[bbx[0]:bbx[2], bbx[3], 1] += color[1]
        blank2[bbx[0], bbx[1]:bbx[3], 1] += color[1]
        blank2[bbx[2], bbx[1]:bbx[3], 1] += color[0]

        blank2[bbx[0]:bbx[2], bbx[1], 2] += color[2]
        blank2[bbx[0]:bbx[2], bbx[3], 2] += color[2]
        blank2[bbx[0], bbx[1]:bbx[3], 2] += color[2]
        blank2[bbx[2], bbx[1]:bbx[3], 2] += color[2]

    except:
        diff = 0.0

    cv2.add(blank, blank2, blank2)
    return blank2


def extract_pos_neg(image, bbxs, size):
    positive = []
    for bb in bbxs:
        xd = bb[3]-bb[1]
        yd = bb[2]-bb[0]
        centerx = bb[1]+xd//2
        centery = bb[0]+yd//2
        m = max(xd//2, yd//2)
        if m >= size*3:
            focus = random.uniform(0.9, 1.1)
            m = int(m * focus)
            crop = image[centerx-m:centerx+m, centery-m:centery+m]
            #print("crop:" , crop.shape[1], " ", crop.shape[0])
            if crop.shape[1]>0 and crop.shape[0]>1:
                positive.append(np.reshape(cv2.resize(crop, (size, size), interpolation=cv2.INTER_CUBIC), [size, size, 1])/255.0)

    label_image = generate_bitmap_labels(image.shape[1], image.shape[0], bbxs)
    negative = []
    sz = len(positive)
    idx = 0
    while idx < sz:
        rx = random.randint(0, image.shape[0] - 2 - size)
        ry = random.randint(0, image.shape[1] - 2 - size)

        r_size = random.randint(size, size*4)
        mean = np.mean(label_image[rx:rx+r_size, ry:ry+r_size])
        if not math.isnan(mean):
            #print("mean: ", mean)
            if mean < 0.001:
                neg_img = image[rx:rx+r_size, ry:ry+r_size]
                neg_img = np.reshape(cv2.resize(neg_img, (size, size), interpolation=cv2.INTER_CUBIC), [size, size, 1])/255.0
                negative.append(neg_img)
                idx += 1

    return positive, negative