from model import model_step1_cnn
from model import model_step2_cnn
from generator_functions import *
import cv2

from get_image import KinectClient
from helpers import navirice_ir_to_np
from helpers import navirice_image_to_np
from position_server import PositionServer
import time

config = tf.ConfigProto()
config.gpu_options.allow_growth=True

def group_adjusted_indexes(adjusted_indexes):
    cpy = adjusted_indexes.copy()
    groups = []
    while(len(cpy) > 0):
        local_group = [cpy[0]]
        local_group_complete = False
        while(not local_group_complete):
            local_group_complete = True
            for i in range(1, len(cpy)):
                if(abs(cpy[0][0]-cpy[i][0]) <= 8 and abs(cpy[0][1]-cpy[i][1]) <= 8):
                    local_group.append(cpy[i])
                    local_group_complete = False
                    del cpy[i]
                    break
        del cpy[0]
        groups.append(local_group)
    return groups

def compute_group_bounds(group):
    x_min = group[0][0]
    y_min = group[0][1]
    x_max = group[0][0]+24
    y_max = group[0][1]+24

    for item in group:
        x_min = min(item[0], x_min)
        y_min = min(item[1], y_min)
        x_max = max(item[0]+24, x_max)
        y_max = max(item[1]+24, y_max)

    return [x_min, y_min, x_max, y_max]

def generate_patch_location_v2(image_shape, stride):
    """Returns a 2d coordinate array of the top left x y locations for each patch."""
    width = image_shape[0]
    height = image_shape[1]
    X, Y = np.mgrid[0:width:stride, 0:height:stride]
    xy = np.dstack((X, Y))
    return xy


def resize_image(image, scale, divisor):
    new = cv2.resize(image.copy(), (0, 0), fx=scale, fy=scale, interpolation=cv2.INTER_CUBIC)
    blank = np.zeros([new.shape[0], new.shape[1], 1], dtype=np.float32)
    blank[:, :, 0] = new
    return blank/divisor

def main():
    x = tf.placeholder(tf.float32, [None, 12, 12, 1])
    x_hidden = tf.placeholder(tf.float32, [None, 16])
    x2 = tf.placeholder(tf.float32, [None, 24, 24, 1])
    y = tf.placeholder(tf.float32, [None, 2])
    y_, y_hidden, variables = model_step1_cnn(x)
    y2_, variables2 = model_step2_cnn(x2, x_hidden)

    gip = tf.placeholder(tf.float32, [1, None, None, 1])
    op = tf.extract_image_patches(gip, ksizes=[1, 12, 12, 1], strides=[1, 2, 2, 1], rates=[1, 1, 1, 1],
                                  padding='VALID')

    op2 = tf.extract_image_patches(gip, ksizes=[1, 24, 24, 1], strides=[1, 4, 4, 1], rates=[1, 1, 1, 1],
                                  padding='VALID')

    sess = tf.Session(config=config)
    init = tf.global_variables_initializer()
    sess.run(init)

    saver = tf.train.Saver(variables)
    try:
        saver.restore(sess, "./model.ckpt")
        print("model1 found")
    except:
        print("no model1 found...")
        exit(1)

    saver2 = tf.train.Saver(variables2)
    try:
        saver2.restore(sess, "./model2.ckpt")
        print("model2 found")
    except:
        print("no model2 found...")
        exit(1)

    kc = KinectClient('192.168.1.129', 29000)
    kc.navirice_capture_settings(False, True, True)
    position_server = PositionServer(4007)

    face_found = False
    crop_values = [0, 0, 0, 0]

    while(True):
        img_set, last_count = kc.navirice_get_image()
        if (img_set != None and img_set.IR.width > 0 and img_set.Depth.width > 0):
            #print("______NEW_IMAGE")
            ir_image = navirice_ir_to_np(img_set.IR) / 255.0

            global_start = time.time()
            start = time.time()
            half_ir_image = resize_image(ir_image, 0.25, 1.0)
            quarter_ir_image = resize_image(half_ir_image, 0.5, 1.0)

            iml_out_ = sess.run(op, feed_dict={gip: [quarter_ir_image]})
            iml_out2_ = sess.run(op2, feed_dict={gip: [half_ir_image]})

            if not face_found:
                crop_values = [0, iml_out_.shape[1], 0, iml_out_.shape[2]]

            original_indexes_ = generate_patch_location_v2([iml_out_.shape[1]*4, iml_out_.shape[2]*4], 4)
            iml_out_ = np.reshape(iml_out_[0], [iml_out_.shape[1], iml_out_.shape[2], 12, 12, 1])
            iml_out2_ = np.reshape(iml_out2_[0], [iml_out2_.shape[1], iml_out2_.shape[2], 24, 24, 1])

            original_indexes_ = original_indexes_[crop_values[0]:crop_values[1], crop_values[2]:crop_values[3]]
            iml_out_ = iml_out_[crop_values[0]:crop_values[1], crop_values[2]:crop_values[3]]
            iml_out2_ = iml_out2_[crop_values[0]:crop_values[1], crop_values[2]:crop_values[3]]

            #print(iml_out_.shape)
            #print(iml_out2_.shape)
            #print(original_indexes_.shape)

            original_indexes_ = np.reshape(original_indexes_, [iml_out_.shape[0]*iml_out_.shape[1], 2])
            iml_out_ = np.reshape(iml_out_, [iml_out_.shape[0]*iml_out_.shape[1], 12, 12, 1])
            iml_out2_ = np.reshape(iml_out2_, [iml_out2_.shape[0]*iml_out2_.shape[1], 24, 24, 1])

            #print(iml_out_.shape)
            #print(iml_out2_.shape)
            #print(original_indexes_.shape)

            end = time.time()
            #print("time elapsed reshape: ", end - start)

            start = time.time()

            confidence, hidden_out = sess.run([y_, y_hidden], feed_dict={x: iml_out_})

            original_indexes_ = original_indexes_[confidence[:, 0] > 0.9]
            iml_out2_ = iml_out2_[confidence[:, 0] > 0.9]
            hidden_out = hidden_out[confidence[:, 0] > 0.9]
            confidence = confidence[confidence[:, 0] > 0.9]

            original_indexes_ = original_indexes_[confidence[:, 1] < 0.1]
            iml_out2_ = iml_out2_[confidence[:, 1] < 0.1]
            hidden_out = hidden_out[confidence[:, 1] < 0.1]
            confidence = confidence[confidence[:, 1] < 0.1]

            #print(original_indexes_.shape)
            end = time.time()
            #print("time elapsed stage1: ", end - start)

            bbx = None

            start = time.time()
            if len(iml_out2_) > 0:
                confidence = sess.run(y2_, feed_dict={x2: iml_out2_, x_hidden: hidden_out})
                original_indexes_ = original_indexes_[confidence[:, 0] > 0.9]
                confidence = confidence[confidence[:, 0] > 0.9]
                original_indexes_ = original_indexes_[confidence[:, 1] < 0.1]
                confidence = confidence[confidence[:, 1] < 0.1]
                if len(original_indexes_) > 0:


                    groups = group_adjusted_indexes(original_indexes_.tolist())
                    bbxs = []
                    for item in groups:
                        bbxs.append(compute_group_bounds(item))


                    lowest_x = 100000000
                    for i in range(len(bbxs)):
                        if (bbxs[i][0] < lowest_x):
                            lowest_x = bbxs[i][0]
                            bbx = bbxs[i]

                    #print(bbx)
                    face_found = True
                    #crop_values = [bbx[0]//4-5, bbx[2]//4+5, bbx[1]//4-5, bbx[3]//4+5]
                    #print(crop_values)

                else:
                    face_found = False

            else:
                face_found = False

            end = time.time()
            #print("time elapsed stage2: ", end - start)




            if bbx is not None:
                bbx[0] = bbx[0] * 4
                bbx[1] = bbx[1] * 4
                bbx[2] = bbx[2] * 4
                bbx[3] = bbx[3] * 4
                bbx_image = draw_bbx(ir_image, bbx, [0, 0, 1])
                cv2.namedWindow("bbx", cv2.WINDOW_GUI_EXPANDED)
                cv2.imshow("bbx", bbx_image)
                start_pos = time.time()
                depth_image = navirice_image_to_np(img_set.Depth)
                send_data_to_positional_server(bbx, position_server, ir_image, depth_image)
                end_pos = time.time()

            else:
                bbx_image = ir_image
                cv2.namedWindow("bbx", cv2.WINDOW_GUI_EXPANDED)
                cv2.imshow("bbx", bbx_image)

            global_end = time.time()
            print("total time: ", global_end - global_start)
            cv2.waitKey(1)


def send_data_to_positional_server(bbx, position_server, ir_image, depth_image):
    #bbx = [bbx[0]*2, bbx[1]*2, bbx[2]*2, bbx[3]*2]
    x = int((bbx[1] + (bbx[3]-bbx[1])/2))
    y = int((bbx[0] + (bbx[2]-bbx[0])/2))
    y = min(y, 423)
    x = min(x, 511)
    depth_render = int(depth_image[y][x] * 5000 * 3)
    #import pdb; pdb.set_trace()
    image_height = ir_image.shape[0]
    image_width = ir_image.shape[1]
    x_01 = x/image_width
    y_01 = y/image_height
    x_render = -((x_01 * 2) - 1)
    y_render  = -((y_01 * 2) - 1)
    #print("bbox: {}".format(bbx))
    #print("xy: {}, {}".format(x, y))
    #print("xy01: {}, {}".format(x_01, y_01))
    #print("xyrender: {}, {}".format(x_render, y_render))
    #print("position data: {}, {}, {}".format(x_render, y_render, depth_render))
    position_server.set_values(x_render, y_render, depth_render)



if __name__ == '__main__':
    main()
